'use strict';

var Promise = require('pinkie-promise');

var _require = require('fetch-ponyfill')({
  Promise: Promise
}),
    fetch = _require.fetch;

var _require2 = require('htmlparser2'),
    DomHandler = _require2.DomHandler,
    DomUtils = _require2.DomUtils,
    Parser = _require2.Parser;

var agent = 'derhuerst/flickr-photo-url'; // Official size suffixes info: https://www.flickr.com/services/api/misc.urls.html

var sizes = ['sq', 'q', 't', 's', 'n', 'm', 'z', 'c', 'l', 'h', 'k', 'o'];
sizes.original = 'o';
sizes.large = 'k';
sizes.medium = 'c';
sizes.small = 'n';
sizes.square = 'q';

var hasProp = function hasProp(o, p) {
  return Object.prototype.hasOwnProperty.call(o, p);
};

var isString = function isString(arg) {
  return typeof arg === 'string';
};

var isNumber = function isNumber(arg) {
  return typeof arg === 'number' || typeof arg === 'bigint';
};

var url = function url(user, id, s) {
  if (!isString(user)) return Promise.reject(new Error('invalid user'));
  if (!isNumber(id)) return Promise.reject(new Error('invalid photo id'));
  if (isNumber(s)) s = sizes[s] || sizes.original;else if (isString(s) && !sizes.includes(s) && !hasProp(sizes, s)) {
    console.error(!sizes.includes(s), !hasProp(sizes, s), s);
    return Promise.reject(new Error('invalid size'));
  }
  return fetch("https://www.flickr.com/photos/".concat(user, "/").concat(id, "/sizes/").concat(s, "/"), {
    redirect: 'follow',
    headers: {
      'user-agent': agent
    }
  }).then(function (res) {
    if (!res.ok) {
      var err = new Error(res.statusText);
      err.statusCode = res.status;
      throw err;
    }

    return res.text();
  }).then(function (html) {
    var dom = parseHtml(html);
    var container = DomUtils.getElementById('allsizes-photo', dom, true);
    var img = DomUtils.findOne(function (el) {
      return el.tagName === 'img';
    }, [container], true);
    var url = img.attribs.src;
    if (!url) throw new Error('failed to extract the URL from the Flickr page');
    return url;
  });
};

module.exports = Object.assign(url, {
  sizes: sizes
});

function parseHtml(html) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var handler = new DomHandler(null, options);
  new Parser(handler, options).end(html);
  return handler.dom;
}
